require("config")

local locomotive = data.raw['locomotive']['locomotive']
local burner = locomotive.burner
burner.fuel_category = "fluid"
burner.fuel_inventory_size = 1

local proxy_straight_rail = table.deepcopy(data.raw['straight-rail']['straight-rail'])
proxy_straight_rail.name = "proxy-straight-rail"
proxy_straight_rail.destructible = nil
proxy_straight_rail.minable = nil
proxy_straight_rail.collision_box = nil
proxy_straight_rail.collision_mask = {}
proxy_straight_rail.selectable_in_game = false
proxy_straight_rail.rail_category = "proxy"
proxy_straight_rail.order = "aaa"
rail_pictures_internal = function(elems)
  local keys = {{"straight_rail", "horizontal", 64, 128, 0, 0, true},
                {"straight_rail", "vertical", 128, 64, 0, 0, true},
                {"straight_rail", "diagonal-left-top", 96, 96, 0.5, 0.5, true},
                {"straight_rail", "diagonal-right-top", 96, 96, -0.5, 0.5, true},
                {"straight_rail", "diagonal-right-bottom", 96, 96, -0.5, -0.5, true},
                {"straight_rail", "diagonal-left-bottom", 96, 96, 0.5, -0.5, true},
                {"curved_rail", "vertical-left-top", 192, 288, 0.5, 0.5},
                {"curved_rail", "vertical-right-top", 192, 288, -0.5, 0.5},
                {"curved_rail", "vertical-right-bottom", 192, 288, -0.5, -0.5},
                {"curved_rail", "vertical-left-bottom", 192, 288, 0.5, -0.5},
                {"curved_rail" ,"horizontal-left-top", 288, 192, 0.5, 0.5},
                {"curved_rail" ,"horizontal-right-top", 288, 192, -0.5, 0.5},
                {"curved_rail" ,"horizontal-right-bottom", 288, 192, -0.5, -0.5},
                {"curved_rail" ,"horizontal-left-bottom", 288, 192, 0.5, -0.5}}
  local res = {}
  for _ , key in ipairs(keys) do
    part = {}
    dashkey = key[1]:gsub("_", "-")
    for _ , elem in ipairs(elems) do
      part[elem[1]] = { sheet = {
        filename = string.format("__base__/graphics/entity/%s/%s-%s-%s.png", dashkey, dashkey, key[2], elem[2]),
        priority = "extra-high",
        flags = elem.mipmap and { "icon" },
        width = key[3],
        height = key[4],
        shift = {key[5], key[6]},
        variation_count = (key[7] and elem.variations) or 1,
		tint = {a = 0},
        hr_version = {
          filename = string.format("__base__/graphics/entity/%s/hr-%s-%s-%s.png", dashkey, dashkey, key[2], elem[2]),
          priority = "extra-high",
          flags = elem.mipmap and { "icon" },
          width = key[3]*2,
          height = key[4]*2,
          shift = {key[5], key[6]},
          scale = 0.5,
          variation_count = (key[7] and elem.variations) or 1,
		  tint = {a = 0}
          }
        }
      }
    end
    dashkey2 = key[2]:gsub("-", "_")
    res[key[1] .. "_" .. dashkey2] = part
  end
    res["rail_endings"] = {
   sheets =
   {
     {
       filename = "__base__/graphics/entity/rail-endings/rail-endings-background.png",
       priority = "high",
       width = 128,
       height = 128,
	   tint = {a = 0},
       hr_version = {
         filename = "__base__/graphics/entity/rail-endings/hr-rail-endings-background.png",
         priority = "high",
         width = 256,
         height = 256,
         scale = 0.5,
		 tint = {a = 0},
       }
     },
     {

       filename = "__base__/graphics/entity/rail-endings/rail-endings-metals.png",
       priority = "high",
       flags = { "icon" },
       width = 128,
       height = 128,
	   tint = {a = 0},
       hr_version = {
         filename = "__base__/graphics/entity/rail-endings/hr-rail-endings-metals.png",
         priority = "high",
         flags = { "icon" },
         width = 256,
         height = 256,
         scale = 0.5,
		 tint = {a = 0},
       }
     }
   }   

 }
  return res
end
local hide_rail_picture = true
proxy_straight_rail.pictures = hide_rail_picture and rail_pictures_internal(
	{
		{"metals", "metals", mipmap = true},
		{"backplates", "backplates", mipmap = true},
		{"ties", "ties", variations = 3},
		{"stone_path", "stone-path", variations = 3},
		{"stone_path_background", "stone-path-background", variations = 3}
	}
) or proxy_straight_rail.pictures

local proxy_tank = table.deepcopy(data.raw['fluid-wagon']['fluid-wagon'])
proxy_tank.name = "proxy-tank"
proxy_tank.total_capacity = TANK_CAPACITY
proxy_tank.destructible = nil
proxy_tank.minable = nil
proxy_tank.selection_box = nil
proxy_tank.operable = false
proxy_tank.order = "z-z-z"
proxy_tank.collision_mask = {}
proxy_tank.rail_category = "proxy"
proxy_tank.selectable_in_game = false
local hide_tank_picture = true
proxy_tank.pictures = hide_tank_picture and {
	layers = {
		{
			priority = "very-low",
			width = 1,
			height = 1,
			back_equals_front = true,
			direction_count = 1,
			filenames = {"__core__/graphics/empty.png"},
			line_length = 1,
			lines_per_file = 1,
			shift = {0 + 0.013, -1 + 0.077},
			hr_version = {
				priority = "very-low",
				width = 1,
				height = 1,
				back_equals_front = true,
				direction_count = 1,
				filenames = {"__core__/graphics/empty.png"},
				line_length = 1,
				lines_per_file = 1,
				shift = {0 + 0.013, -1 + 0.077},
				scale = 0.5
			}
		},
		{
			flags = { "shadow" },
			priority = "very-low",
			width = 1,
			height = 1,
			back_equals_front = true,
			draw_as_shadow = true,
			direction_count = 1,
			filenames = {"__core__/graphics/empty.png"},
			line_length = 1,
			lines_per_file = 1,
			shift = {0.875 + 0.013, 0.3125 + 0.077},
			hr_version = {
				flags = { "shadow" },
				priority = "very-low",
				width = 1,
				height = 1,
				back_equals_front = true,
				draw_as_shadow = true,
				direction_count = 1,
				filenames = {"__core__/graphics/empty.png"},
				line_length = 1,
				lines_per_file = 1,
				shift = {0.875 + 0.013, 0.3125 + 0.077},
				scale = 0.5
			}
		}
	}
} or proxy_tank.pictures

data:extend({proxy_straight_rail, proxy_tank})